from django.contrib.admin import ModelAdmin

class DdcuAdminMixin(ModelAdmin):
    class Media:
        css = {'all': ("direct_cloud_upload/cloud_file_widget.css", )}
        js = ("direct_cloud_upload/ddcu_upload.js", )



import json
import time

from django.forms import Widget
from django.urls import reverse
from django.utils import baseconv
from django.core.signing import Signer

signer = Signer()


def get_filename_from_path(path: str) -> str:
    return path.rsplit('/', 1)[-1] if path else ""


class CloudFileWidget(Widget):
    template_name = "cloud_file_widget.html"
    def __init__(self,
                 bucket_identifier: str,
                 path_prefix: str = "",
                 include_timestamp: bool = True,
                 submit_timeout: int = 36 * 3600,
                 clearable: bool = True,
                 immediate_submit: bool = False,
                 show_replace_button: bool = True,
                 allow_multiple: bool = False,
                 *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.bucket_identifier = bucket_identifier
        self.path_prefix = path_prefix
        self.include_timestamp = include_timestamp
        self.submit_timeout = submit_timeout
        self.clearable = clearable
        self.immediate_submit = immediate_submit
        self.show_replace_button = show_replace_button
        self.allow_multiple = allow_multiple

    def get_context(self, name, value, attrs):
        context = super(CloudFileWidget, self).get_context(name, value, attrs)
        include_timestamp_indicator = '1' if self.include_timestamp else '0'
        allow_multiple_indicator = '1' if self.allow_multiple else '0'
        valid_until = baseconv.base62.encode(int(time.time()) + self.submit_timeout)
        to_sign = f"{self.bucket_identifier}/{self.path_prefix}:{include_timestamp_indicator}:{allow_multiple_indicator}:{valid_until}"
        context['ddcu_token'] = signer.sign(to_sign)
        context['guu_path'] = reverse('ddcu-get-upload-url')
        context['input_clearable'] = self.clearable
        context['immediate_submit'] = self.immediate_submit
        context['show_replace_button'] = self.show_replace_button
        context['allow_multiple'] = self.allow_multiple
        if self.allow_multiple:
            if value:
                if value.startswith('['):
                    paths = json.loads(value)
                else:
                    paths = [value]
            else:
                paths = []
            context['existing_files'] = [(path, get_filename_from_path(path)) for path in paths]
        else:
            context['current_file_name'] = get_filename_from_path(value)
        return context


import random, string, json, datetime, time

from django.http import HttpResponseBadRequest, HttpResponse
from django.views.decorators.http import require_POST
from django.utils import timezone, baseconv
import django.core.signing
from google.cloud.storage import Blob, Bucket

from .bucket_registry import _bucket_registry


URLSAFE_CHARACTERS = string.ascii_letters + string.digits + "-._~"
signer = django.core.signing.Signer()


@require_POST
def get_upload_url(request):
    if not request.POST.get('token'):
        return HttpResponseBadRequest(f"'token' is a required parameter.")
    try:
        token: str = signer.unsign(request.POST['token'])
    except django.core.signing.BadSignature:
        return HttpResponseBadRequest("Invalid token.")

    bucket_and_path, include_timestamp_indicator, allow_multiple_indicator, exptime = token.rsplit(':', 3)
    if time.time() > baseconv.base62.decode(exptime):
        return HttpResponseBadRequest("Timeout expired.")
    timestring: str = "{0:%Y-%m-%d_%H-%M-%S/}".format(timezone.now()) if include_timestamp_indicator == '1' else ""

    bucketname, path_prefix = bucket_and_path[5:].split('/', 1)
    bucket: Bucket = _bucket_registry.get('gs://' + bucketname)
    if not bucket:
        return HttpResponseBadRequest(f"Unknown bucket identifier 'gs://{bucketname}'.")

    counter = 0
    urls = []
    paths = []
    while f'filename_{counter}' in request.POST and f'content_type_{counter}' in request.POST:
        randomstring: str = "".join(random.choices(URLSAFE_CHARACTERS, k=24))
        path: str = f"{path_prefix}{timestring}{randomstring}/{request.POST[f'filename_{counter}']}"
        blob: Blob = bucket.blob(path)
        urls.append(blob.generate_signed_url(
            expiration = timezone.now() + datetime.timedelta(seconds=60),
            method = 'PUT',
            content_type = request.POST[f'content_type_{counter}'],
        ))
        paths.append(path)
        counter += 1

    if allow_multiple_indicator == '0':
        inputval = paths[0]
    elif allow_multiple_indicator == '1':
        inputval = json.dumps(paths)
    else:
        raise ValueError(f"Invalid allow_multiple_indicator '{allow_multiple_indicator}'.")

    return HttpResponse(json.dumps({
        'urls': urls,
        'inputval': inputval,
    }))


from django.urls import path

from .widgets import CloudFileWidget
from .views import get_upload_url
from .admin import DdcuAdminMixin
from .bucket_registry import register_gcs_bucket

urlpatterns = [path("get_upload_url", get_upload_url, name='ddcu-get-upload-url')]

__all__ = [
    'CloudFileWidget', 'DdcuAdminMixin', 'urlpatterns', 'register_gcs_bucket',
]

default_app_config = "direct_cloud_upload.apps.DdcuConfig"


from typing import Dict
from google.cloud.storage import Bucket

_bucket_registry: Dict[str, Bucket] = dict()

def register_gcs_bucket(bucket):
    id = "gs://" + bucket.name
    _bucket_registry[id] = bucket
    return id